@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2024 Hans Petter Jansson
+/* Copyright (C) 2018-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -110,6 +110,7 @@ Glyph2;
  * @CHAFA_SYMBOL_TAG_WEDGE: Wedge shapes that align with sextants.
  * @CHAFA_SYMBOL_TAG_LATIN: Latin and Latin-like symbols (superset of ASCII).
  * @CHAFA_SYMBOL_TAG_IMPORTED: Symbols for which glyphs were imported with chafa_symbol_map_add_glyph ().
+ * @CHAFA_SYMBOL_TAG_OCTANT: Octant 2x4 mosaics.
  * @CHAFA_SYMBOL_TAG_EXTRA: Symbols not in any other category.
  * @CHAFA_SYMBOL_TAG_BAD: Joint set of ugly and ambiguous characters. Always excluded unless specifically asked for.
  * @CHAFA_SYMBOL_TAG_ALL: Special value meaning all supported symbols.
@@ -869,6 +870,7 @@ parse_symbol_tag (const gchar *name, gint len, SelectorType *sel_type_out,
         { "latin", CHAFA_SYMBOL_TAG_LATIN },
         { "import", CHAFA_SYMBOL_TAG_IMPORTED },
         { "imported", CHAFA_SYMBOL_TAG_IMPORTED },
+        { "octant", CHAFA_SYMBOL_TAG_OCTANT },
 
         { NULL, 0 }
     };
