@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2024 Hans Petter Jansson
+/* Copyright (C) 2019-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -57,7 +57,7 @@ void chafa_palette_deinit (ChafaPalette *palette);
 
 void chafa_palette_copy (const ChafaPalette *src, ChafaPalette *dest);
 void chafa_palette_generate (ChafaPalette *palette_out, gconstpointer pixels, gint n_pixels,
-                             ChafaColorSpace color_space);
+                             ChafaColorSpace color_space, gfloat quality);
 
 ChafaPaletteType chafa_palette_get_type (const ChafaPalette *palette);
 
