@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2020-2024 Hans Petter Jansson
+/* Copyright (C) 2020-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -19,8 +19,10 @@
 
 #include "config.h"
 
+#include "chafa.h"
 #include "internal/chafa-batch.h"
 #include "internal/chafa-pixops.h"
+#include "internal/chafa-math-util.h"
 #include "internal/smolscale/smolscale.h"
 
 /* Fixed point multiplier */
@@ -293,7 +295,7 @@ convert_rgb_to_din99d (ChafaPixel *pixels, gint width, gint dest_y, gint n_rows)
 }
 
 static void
-bayer_dither (const ChafaDither *dither, ChafaPixel *pixels, gint width, gint dest_y, gint n_rows)
+simple_dither (const ChafaDither *dither, ChafaPixel *pixels, gint width, gint dest_y, gint n_rows)
 {
     ChafaPixel *pixel = pixels + dest_y * width;
     ChafaPixel *pixel_max = pixel + n_rows * width;
@@ -303,7 +305,7 @@ bayer_dither (const ChafaDither *dither, ChafaPixel *pixels, gint width, gint de
     {
         for (x = 0; x < width; x++)
         {
-            pixel->col = chafa_dither_color_ordered (dither, pixel->col, x, y);
+            pixel->col = chafa_dither_color (dither, pixel->col, x, y);
             pixel++;
         }
     }
@@ -413,8 +415,8 @@ fs_dither (const ChafaDither *dither, const ChafaPalette *palette,
 }
 
 static void
-bayer_and_convert_rgb_to_din99d (const ChafaDither *dither,
-                                 ChafaPixel *pixels, gint width, gint dest_y, gint n_rows)
+dither_and_convert_rgb_to_din99d (const ChafaDither *dither,
+                                  ChafaPixel *pixels, gint width, gint dest_y, gint n_rows)
 {
     ChafaPixel *pixel = pixels + dest_y * width;
     ChafaPixel *pixel_max = pixel + n_rows * width;
@@ -424,7 +426,7 @@ bayer_and_convert_rgb_to_din99d (const ChafaDither *dither,
     {
         for (x = 0; x < width; x++)
         {
-            pixel->col = chafa_dither_color_ordered (dither, pixel->col, x, y);
+            pixel->col = chafa_dither_color (dither, pixel->col, x, y);
             chafa_color_rgb_to_din99d (&pixel->col, &pixel->col);
             pixel++;
         }
@@ -485,7 +487,7 @@ prepare_pixels_1_worker_nearest (ChafaBatchInfo *batch, PrepareContext *prep_ctx
     gint rowstride;
     PreparePixelsBatch1Ret *ret;
 
-    ret = g_new (PreparePixelsBatch1Ret, 1);
+    ret = g_new0 (PreparePixelsBatch1Ret, 1);
     batch->ret_p = ret;
 
     dest_y = batch->first_row;
@@ -643,13 +645,14 @@ prepare_pixels_2_worker (ChafaBatchInfo *batch, PrepareContext *prep_ctx)
 
     if (prep_ctx->color_space == CHAFA_COLOR_SPACE_DIN99D)
     {
-        if (prep_ctx->dither->mode == CHAFA_DITHER_MODE_ORDERED)
+        if (prep_ctx->dither->mode == CHAFA_DITHER_MODE_ORDERED
+            || prep_ctx->dither->mode == CHAFA_DITHER_MODE_NOISE)
         {
-            bayer_and_convert_rgb_to_din99d (prep_ctx->dither,
-                                             prep_ctx->dest_pixels,
-                                             prep_ctx->dest_width,
-                                             batch->first_row,
-                                             batch->n_rows);
+            dither_and_convert_rgb_to_din99d (prep_ctx->dither,
+                                              prep_ctx->dest_pixels,
+                                              prep_ctx->dest_width,
+                                              batch->first_row,
+                                              batch->n_rows);
         }
         else if (prep_ctx->dither->mode == CHAFA_DITHER_MODE_DIFFUSION)
         {
@@ -668,13 +671,14 @@ prepare_pixels_2_worker (ChafaBatchInfo *batch, PrepareContext *prep_ctx)
                                    batch->n_rows);
         }
     }
-    else if (prep_ctx->dither->mode == CHAFA_DITHER_MODE_ORDERED)
+    else if (prep_ctx->dither->mode == CHAFA_DITHER_MODE_ORDERED
+             || prep_ctx->dither->mode == CHAFA_DITHER_MODE_NOISE)
     {
-        bayer_dither (prep_ctx->dither,
-                      prep_ctx->dest_pixels,
-                      prep_ctx->dest_width,
-                      batch->first_row,
-                      batch->n_rows);
+        simple_dither (prep_ctx->dither,
+                       prep_ctx->dest_pixels,
+                       prep_ctx->dest_width,
+                       batch->first_row,
+                       batch->n_rows);
     }
     else if (prep_ctx->dither->mode == CHAFA_DITHER_MODE_DIFFUSION)
     {
@@ -752,9 +756,48 @@ chafa_prepare_pixel_data_for_symbols (const ChafaPalette *palette,
                                       gint src_rowstride,
                                       ChafaPixel *dest_pixels,
                                       gint dest_width,
-                                      gint dest_height)
+                                      gint dest_height,
+                                      gint cell_width,
+                                      gint cell_height,
+                                      ChafaAlign halign,
+                                      ChafaAlign valign,
+                                      ChafaTuck tuck)
 {
     PrepareContext prep_ctx = { 0 };
+    gint placement_x, placement_y;
+    gint placement_width, placement_height;
+
+    /* Convert the destination dimensions from symbol matrix geometry to real
+     * geometry (just for the calculation) for correct image sizing. */
+    chafa_tuck_and_align (src_width, src_height,
+                          (dest_width / CHAFA_SYMBOL_WIDTH_PIXELS) * cell_width,
+                          (dest_height / CHAFA_SYMBOL_HEIGHT_PIXELS) * cell_height,
+                          halign, valign,
+                          tuck,
+                          &placement_x, &placement_y,
+                          &placement_width, &placement_height);
+
+    /* Rounding the placement edges to cell boundaries prevents artifacts
+     * in the first/last row/col containing the actual image,
+     * when tuck == FIT or SHRINK_TO_FIT. */
+
+    /* First image row/col rounds *down* to the nearest cell boundary. */
+    placement_x -= placement_x % cell_width;
+    placement_y -= placement_y % cell_height;
+
+    /* Last image row/col rounds *up* to the nearest cell boundary.
+     *
+     * Note: If the left/top edge is on a cell boundary (which it already is),
+     * and the width/height is a multiple of the cell width/height,
+     * then the right/bottom edge is also on a cell boundary. */
+    placement_width = chafa_round_up_to_multiple_of (placement_width, cell_width);
+    placement_height = chafa_round_up_to_multiple_of (placement_height, cell_height);
+
+    /* Convert the placement dimensions from real geometry to symbol matrix geometry. */
+    placement_x = (placement_x / cell_width) * CHAFA_SYMBOL_WIDTH_PIXELS,
+    placement_y = (placement_y / cell_height) * CHAFA_SYMBOL_HEIGHT_PIXELS,
+    placement_width = (placement_width / cell_width) * CHAFA_SYMBOL_WIDTH_PIXELS,
+    placement_height = (placement_height / cell_height) * CHAFA_SYMBOL_HEIGHT_PIXELS,
 
     prep_ctx.palette = palette;
     prep_ctx.dither = dither;
@@ -777,17 +820,31 @@ chafa_prepare_pixel_data_for_symbols (const ChafaPalette *palette,
     prep_ctx.dest_width = dest_width;
     prep_ctx.dest_height = dest_height;
 
-    prep_ctx.scale_ctx = smol_scale_new_simple (prep_ctx.src_pixels,
-                                                (SmolPixelType) prep_ctx.src_pixel_type,
-                                                prep_ctx.src_width,
-                                                prep_ctx.src_height,
-                                                prep_ctx.src_rowstride,
-                                                NULL,
-                                                SMOL_PIXEL_RGBA8_UNASSOCIATED,  /* FIXME: Premul */
-                                                prep_ctx.dest_width,
-                                                prep_ctx.dest_height,
-                                                prep_ctx.dest_width * sizeof (guint32),
-                                                SMOL_NO_FLAGS);
+    prep_ctx.scale_ctx = smol_scale_new_full (/* Source */
+                                              prep_ctx.src_pixels,
+                                              (SmolPixelType) prep_ctx.src_pixel_type,
+                                              prep_ctx.src_width,
+                                              prep_ctx.src_height,
+                                              prep_ctx.src_rowstride,
+                                              /* Fill */
+                                              NULL,
+                                              SMOL_PIXEL_RGBA8_UNASSOCIATED,
+                                              /* Destination */
+                                              NULL,
+                                              SMOL_PIXEL_RGBA8_UNASSOCIATED,  /* FIXME: Premul */
+                                              prep_ctx.dest_width,
+                                              prep_ctx.dest_height,
+                                              prep_ctx.dest_width * sizeof (guint32),
+                                              /* Placement */
+                                              placement_x * SMOL_SUBPIXEL_MUL,
+                                              placement_y * SMOL_SUBPIXEL_MUL,
+                                              placement_width * SMOL_SUBPIXEL_MUL,
+                                              placement_height * SMOL_SUBPIXEL_MUL,
+                                              /* Extra args */
+                                              SMOL_COMPOSITE_SRC_CLEAR_DEST,
+                                              SMOL_NO_FLAGS,
+                                              NULL,
+                                              &prep_ctx);
 
     prepare_pixels_pass_1 (&prep_ctx);
     prepare_pixels_pass_2 (&prep_ctx);
