@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2024 Hans Petter Jansson
+/* Copyright (C) 2019-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -42,7 +42,8 @@ void chafa_sixel_canvas_draw_all_pixels (ChafaSixelCanvas *sixel_canvas, ChafaPi
                                          gconstpointer src_pixels,
                                          gint src_width, gint src_height, gint src_rowstride,
                                          ChafaAlign halign, ChafaAlign valign,
-                                         ChafaTuck tuck);
+                                         ChafaTuck tuck,
+                                         gfloat quality);
 void chafa_sixel_canvas_build_ansi (ChafaSixelCanvas *sixel_canvas, ChafaTermInfo *term_info,
                                     GString *out_str, ChafaPassthrough passthrough);
 
