@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2024 Hans Petter Jansson
+/* Copyright (C) 2018-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -23,6 +23,62 @@
  * This is meant to be #included in the symbol definition table of
  * chafa-symbols.c. It's kept in a separate file due to its size. */
 
+    {
+        /* Leftwards arrow */
+        CHAFA_SYMBOL_TAG_TECHNICAL,
+        0x2190,
+        CHAFA_SYMBOL_OUTLINE_8X8 (
+            "        "
+            "   X    "
+            "  XX    "
+            " XXXXXX "
+            "  XX    "
+            "   X    "
+            "        "
+            "        ")
+    },
+    {
+        /* Upwards arrow */
+        CHAFA_SYMBOL_TAG_TECHNICAL,
+        0x2191,
+        CHAFA_SYMBOL_OUTLINE_8X8 (
+            "        "
+            "   X    "
+            "  XXX   "
+            " XXXXX  "
+            "   X    "
+            "   X    "
+            "   X    "
+            "        ")
+    },
+    {
+        /* Rightwards arrow */
+        CHAFA_SYMBOL_TAG_TECHNICAL,
+        0x2192,
+        CHAFA_SYMBOL_OUTLINE_8X8 (
+            "        "
+            "    X   "
+            "    XX  "
+            " XXXXXX "
+            "    XX  "
+            "    X   "
+            "        "
+            "        ")
+    },
+    {
+        /* Downwards arrow */
+        CHAFA_SYMBOL_TAG_TECHNICAL,
+        0x2193,
+        CHAFA_SYMBOL_OUTLINE_8X8 (
+            "        "
+            "   X    "
+            "   X    "
+            "   X    "
+            " XXXXX  "
+            "  XXX   "
+            "   X    "
+            "        ")
+    },
     {
         /* Horizontal Scan Line 1 */
         CHAFA_SYMBOL_TAG_TECHNICAL,
